@@ -1,14 +1,18 @@
 try:
-    from cffi import FFI
+    import cffi
 except ImportError:
     # PyPy includes cffi by default
-    msg = ('To build the CFFI implementation of pyzstd module, need to '
-           'install cffi module like this: "sudo python3 -m pip install '
-           'cffi". On CPython, CFFI implementation is slower than C '
-           'implementation.')
-    raise ImportError(msg)
+    msg = ('\n    To build the CFFI implementation '
+           'of pyzstd module, need cffi module.'
+           '\n    On CPython, CFFI implementation '
+           'is slower than C implementation.\n')
+    print(msg)
 
-ffibuilder = FFI()
+    # Used for PEP-517 get_requires_for_build_wheel hook
+    from setuptools.build_meta import SetupRequirementsError
+    raise SetupRequirementsError(['cffi'])
+
+ffibuilder = cffi.FFI()
 
 ffibuilder.cdef("""
 #define ZSTD_VERSION_NUMBER ...
@@ -123,12 +127,17 @@ size_t ZSTD_DStreamOutSize(void);
 ZSTD_CDict* ZSTD_createCDict(const void* dictBuffer, size_t dictSize,
                              int compressionLevel);
 size_t ZSTD_CCtx_refCDict(ZSTD_CCtx* cctx, const ZSTD_CDict* cdict);
+size_t ZSTD_CCtx_loadDictionary(ZSTD_CCtx* cctx, const void* dict, size_t dictSize);
 size_t ZSTD_freeCDict(ZSTD_CDict* CDict);
+size_t ZSTD_CCtx_refPrefix(ZSTD_CCtx* cctx, const void* prefix, size_t prefixSize);
 
 ZSTD_DDict* ZSTD_createDDict(const void* dictBuffer, size_t dictSize);
 size_t ZSTD_DCtx_refDDict(ZSTD_DCtx* dctx, const ZSTD_DDict* ddict);
+size_t ZSTD_DCtx_loadDictionary(ZSTD_DCtx* dctx, const void* dict, size_t dictSize);
 size_t ZSTD_freeDDict(ZSTD_DDict* ddict);
-unsigned ZSTD_getDictID_fromDDict(const ZSTD_DDict* ddict);
+size_t ZSTD_DCtx_refPrefix(ZSTD_DCtx* dctx, const void* prefix, size_t prefixSize);
+
+unsigned ZSTD_getDictID_fromDict(const void* dict, size_t dictSize);
 
 ZSTD_CCtx* ZSTD_createCCtx(void);
 size_t ZSTD_freeCCtx(ZSTD_CCtx* cctx);
@@ -162,6 +171,8 @@ size_t ZDICT_finalizeDictionary(void* dstDictBuffer, size_t maxDictSize,
                                 const void* dictContent, size_t dictContentSize,
                                 const void* samplesBuffer, const size_t* samplesSizes, unsigned nbSamples,
                                 ZDICT_params_t parameters);
+
+extern int pyzstd_static_link;
 """)
 
 source = """
@@ -194,6 +205,12 @@ int ZSTD_defaultCLevel(void)
     return ZSTD_CLEVEL_DEFAULT;
 }
 #endif
+
+#ifdef PYZSTD_STATIC_LINK
+int pyzstd_static_link = 1;
+#else
+int pyzstd_static_link = 0;
+#endif
 """
 
 def get_extension(**kwargs):
